@@ -1,18 +1,15 @@
-var PoltergeistAgent;
+var PoltergeistAgent,
+  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
+  hasProp = {}.hasOwnProperty;
 
 PoltergeistAgent = (function() {
-  PoltergeistAgent.JSON || (PoltergeistAgent.JSON = {
-    parse: JSON.parse,
-    stringify: JSON.stringify
-  });
-
   function PoltergeistAgent() {
     this.elements = [];
     this.nodes = {};
   }
 
   PoltergeistAgent.prototype.externalCall = function(name, args) {
-    var error, error1;
+    var error;
     try {
       return {
         value: this[name].apply(this, args)
@@ -28,32 +25,12 @@ PoltergeistAgent = (function() {
     }
   };
 
-  PoltergeistAgent.stringify = function(object) {
-    var error, error1;
-    try {
-      return PoltergeistAgent.JSON.stringify(object, function(key, value) {
-        if (Array.isArray(this[key])) {
-          return this[key];
-        } else {
-          return value;
-        }
-      });
-    } catch (error1) {
-      error = error1;
-      if (error instanceof TypeError) {
-        return '"(cyclic structure)"';
-      } else {
-        throw error;
-      }
-    }
-  };
-
-  PoltergeistAgent.prototype.currentUrl = function() {
-    return window.location.href.replace(/\ /g, '%20');
+  PoltergeistAgent.prototype.frameUrl = function() {
+    return window.location.href;
   };
 
   PoltergeistAgent.prototype.find = function(method, selector, within) {
-    var el, error, error1, i, j, len, results, results1, xpath;
+    var el, error, i, j, len, results, results1, xpath;
     if (within == null) {
       within = document;
     }
@@ -122,7 +99,50 @@ PoltergeistAgent = (function() {
   };
 
   PoltergeistAgent.prototype.clearLocalStorage = function() {
-    return localStorage.clear();
+    var error;
+    try {
+      return typeof localStorage !== "undefined" && localStorage !== null ? localStorage.clear() : void 0;
+    } catch (error1) {
+      error = error1;
+    }
+  };
+
+  PoltergeistAgent.prototype.wrapResults = function(result, page_id) {
+    var j, key, len, obj, res, results1, val;
+    this._visitedObjects || (this._visitedObjects = []);
+    switch (false) {
+      case indexOf.call(this._visitedObjects, result) < 0:
+        return '(cyclic structure)';
+      case !(Array.isArray(result) || (result instanceof NodeList)):
+        results1 = [];
+        for (j = 0, len = result.length; j < len; j++) {
+          res = result[j];
+          results1.push(this.wrapResults(res, page_id));
+        }
+        return results1;
+        break;
+      case !(result && result.nodeType === 1 && result['tagName']):
+        return {
+          'ELEMENT': {
+            id: this.register(result),
+            page_id: page_id
+          }
+        };
+      case !(result == null):
+        return void 0;
+      case typeof result !== 'object':
+        this._visitedObjects.push(result);
+        obj = {};
+        for (key in result) {
+          if (!hasProp.call(result, key)) continue;
+          val = result[key];
+          obj[key] = this.wrapResults(val, page_id);
+        }
+        this._visitedObjects.pop();
+        return obj;
+      default:
+        return result;
+    }
   };
 
   return PoltergeistAgent;
@@ -187,7 +207,7 @@ PoltergeistAgent.Node = (function() {
     obsolete = (function(_this) {
       return function(element) {
         var parent;
-        if ((parent = element.parentNode) != null) {
+        if ((parent = element != null ? element.parentNode : void 0) != null) {
           if (parent === document) {
             return false;
           } else {
@@ -296,10 +316,11 @@ PoltergeistAgent.Node = (function() {
   };
 
   Node.prototype.getAttribute = function(name) {
+    var ref;
     if (name === 'checked' || name === 'selected') {
       return this.element[name];
     } else {
-      return this.element.getAttribute(name);
+      return (ref = this.element.getAttribute(name)) != null ? ref : void 0;
     }
   };
 
@@ -372,10 +393,10 @@ PoltergeistAgent.Node = (function() {
     } else if (value === false && !this.element.parentNode.multiple) {
       return false;
     } else {
-      this.trigger('focus', this.element.parentNode);
+      this.trigger('focus', {}, this.element.parentNode);
       this.element.selected = value;
       this.changed();
-      this.trigger('blur', this.element.parentNode);
+      this.trigger('blur', {}, this.element.parentNode);
       return true;
     }
   };
@@ -413,7 +434,9 @@ PoltergeistAgent.Node = (function() {
   };
 
   Node.prototype.isDisabled = function() {
-    return this.element.disabled || this.element.tagName === 'OPTION' && this.element.parentNode.disabled;
+    var xpath;
+    xpath = 'parent::optgroup[@disabled] | ancestor::select[@disabled] | parent::fieldset[@disabled] | ancestor::*[not(self::legend) or preceding-sibling::legend][parent::fieldset[@disabled]]';
+    return this.element.disabled || document.evaluate(xpath, this.element, null, XPathResult.BOOLEAN_TYPE, null).booleanValue;
   };
 
   Node.prototype.path = function() {
@@ -479,14 +502,17 @@ PoltergeistAgent.Node = (function() {
     return pos;
   };
 
-  Node.prototype.trigger = function(name, element) {
+  Node.prototype.trigger = function(name, options, element) {
     var event;
+    if (options == null) {
+      options = {};
+    }
     if (element == null) {
       element = this.element;
     }
     if (Node.EVENTS.MOUSE.indexOf(name) !== -1) {
       event = document.createEvent('MouseEvent');
-      event.initMouseEvent(name, true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
+      event.initMouseEvent(name, true, true, window, 0, options['screenX'] || 0, options['screenY'] || 0, options['clientX'] || 0, options['clientY'] || 0, options['ctrlKey'] || false, options['altKey'] || false, options['shiftKey'] || false, options['metaKey'] || false, options['button'] || 0, null);
     } else if (Node.EVENTS.FOCUS.indexOf(name) !== -1) {
       event = this.obtainEvent(name);
     } else if (Node.EVENTS.FORM.indexOf(name) !== -1) {
@@ -596,3 +622,7 @@ window.__poltergeist = new PoltergeistAgent;
 document.addEventListener('DOMContentLoaded', function() {
   return console.log('__DOMContentLoaded');
 });
+
+if (document.readyState === 'complete') {
+  console.log('__DOMContentLoaded');
+}
