@@ -1,5 +1,5 @@
-var indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
-  slice = [].slice;
+var slice = [].slice,
+  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };
 
 Poltergeist.Browser = (function() {
   function Browser(width, height) {
@@ -9,6 +9,7 @@ Poltergeist.Browser = (function() {
     this.js_errors = true;
     this._debug = false;
     this._counter = 0;
+    this._page_settings = null;
     this.processed_modal_messages = [];
     this.confirm_processes = [];
     this.prompt_responses = [];
@@ -20,14 +21,14 @@ Poltergeist.Browser = (function() {
     ref = [0, []], this._counter = ref[0], this.pages = ref[1];
     if (this.page != null) {
       if (!this.page.closed) {
-        if (this.page.currentUrl() !== 'about:blank') {
+        if (this.page.frameUrl() !== 'about:blank') {
           this.page.clearLocalStorage();
         }
-        this.page.release();
+        this.page.close();
       }
       phantom.clearCookies();
     }
-    this.page = this.currentPage = new Poltergeist.WebPage;
+    this.page = this.currentPage = new Poltergeist.WebPage(null, this._page_settings);
     this.page.setViewportSize({
       width: this.width,
       height: this.height
@@ -37,12 +38,16 @@ Poltergeist.Browser = (function() {
     this.processed_modal_messages = [];
     this.confirm_processes = [];
     this.prompt_responses = [];
-    this.page["native"]().onAlert = (function(_this) {
+    this.setupPageHandlers(this.page);
+  };
+
+  Browser.prototype.setupPageHandlers = function(page) {
+    page["native"]().onAlert = (function(_this) {
       return function(msg) {
         _this.setModalMessage(msg);
       };
     })(this);
-    this.page["native"]().onConfirm = (function(_this) {
+    page["native"]().onConfirm = (function(_this) {
       return function(msg) {
         var process;
         process = _this.confirm_processes.pop();
@@ -53,7 +58,7 @@ Poltergeist.Browser = (function() {
         return process;
       };
     })(this);
-    this.page["native"]().onPrompt = (function(_this) {
+    page["native"]().onPrompt = (function(_this) {
       return function(msg, defaultVal) {
         var response;
         response = _this.prompt_responses.pop();
@@ -64,15 +69,18 @@ Poltergeist.Browser = (function() {
         return response;
       };
     })(this);
-    this.page.onPageCreated = (function(_this) {
+    page.onPageCreated = (function(_this) {
       return function(newPage) {
-        var page;
-        page = new Poltergeist.WebPage(newPage);
-        page.handle = "" + (_this._counter++);
-        page.urlBlacklist = _this.page.urlBlacklist;
-        page.urlWhitelist = _this.page.urlWhitelist;
-        page.setViewportSize(_this.page.viewportSize());
-        return _this.pages.push(page);
+        var _page;
+        _page = new Poltergeist.WebPage(newPage, _this._page_settings);
+        _page.handle = "" + (_this._counter++);
+        _page.urlBlacklist = page.urlBlacklist;
+        _page.urlWhitelist = page.urlWhitelist;
+        _page.setViewportSize(page.viewportSize());
+        _page.setUserAgent(page.getUserAgent());
+        _page.setCustomHeaders(page.getPermanentCustomHeaders());
+        _this.setupPageHandlers(_page);
+        return _this.pages.push(_page);
       };
     })(this);
   };
@@ -100,8 +108,11 @@ Poltergeist.Browser = (function() {
   };
 
   Browser.prototype.add_extension = function(extension) {
-    this.currentPage.injectExtension(extension);
-    return this.current_command.sendResponse('success');
+    if (this.currentPage.injectExtension(extension)) {
+      return this.current_command.sendResponse('success');
+    } else {
+      return this.current_command.sendError(new Poltergeist.BrowserError("Unable to load extension: " + extension));
+    }
   };
 
   Browser.prototype.node = function(page_id, id) {
@@ -121,7 +132,7 @@ Poltergeist.Browser = (function() {
     this.processed_modal_messages = [];
     this.confirm_processes = [];
     this.prompt_responses = [];
-    prevUrl = this.currentPage.source === null ? 'about:blank' : this.currentPage.currentUrl();
+    prevUrl = this.currentPage.source != null ? this.currentPage.currentUrl() : 'about:blank';
     this.currentPage.open(url);
     if (/#/.test(url) && prevUrl.split('#')[0] === url.split('#')[0]) {
       this.currentPage.state = 'default';
@@ -142,7 +153,7 @@ Poltergeist.Browser = (function() {
       }, max_wait, function() {
         var msg, resources;
         resources = this.openResourceRequests();
-        msg = resources.length ? "Timed out with the following resources still waiting " + (resources.join(',')) : void 0;
+        msg = resources.length ? "Timed out with the following resources still waiting " + (resources.join(',')) : "Timed out with no open resource requests";
         return command.sendError(new Poltergeist.StatusFailError(url, msg));
       });
     }
@@ -152,6 +163,10 @@ Poltergeist.Browser = (function() {
     return this.current_command.sendResponse(this.currentPage.currentUrl());
   };
 
+  Browser.prototype.frame_url = function() {
+    return this.current_command.sendResponse(this.currentPage.frameUrl());
+  };
+
   Browser.prototype.status_code = function() {
     return this.current_command.sendResponse(this.currentPage.statusCode);
   };
@@ -168,6 +183,10 @@ Poltergeist.Browser = (function() {
     return this.current_command.sendResponse(this.currentPage.title());
   };
 
+  Browser.prototype.frame_title = function() {
+    return this.current_command.sendResponse(this.currentPage.frameTitle());
+  };
+
   Browser.prototype.find = function(method, selector) {
     return this.current_command.sendResponse({
       page_id: this.currentPage.id,
@@ -249,17 +268,62 @@ Poltergeist.Browser = (function() {
     return this.current_command.sendResponse(this.node(page_id, id).path());
   };
 
-  Browser.prototype.evaluate = function(script) {
-    return this.current_command.sendResponse(this.currentPage.evaluate("function() { return " + script + " }"));
+  Browser.prototype.evaluate = function() {
+    var arg, args, i, len, ref, script;
+    script = arguments[0], args = 2 <= arguments.length ? slice.call(arguments, 1) : [];
+    for (i = 0, len = args.length; i < len; i++) {
+      arg = args[i];
+      if (this._isElementArgument(arg)) {
+        if (arg["ELEMENT"]["page_id"] !== this.currentPage.id) {
+          throw new Poltergeist.ObsoleteNode;
+        }
+      }
+    }
+    return this.current_command.sendResponse((ref = this.currentPage).evaluate.apply(ref, ["function() { return " + script + " }"].concat(slice.call(args))));
   };
 
-  Browser.prototype.execute = function(script) {
-    this.currentPage.execute("function() { " + script + " }");
+  Browser.prototype.evaluate_async = function() {
+    var arg, args, cb, command, i, len, max_wait, ref, script;
+    script = arguments[0], max_wait = arguments[1], args = 3 <= arguments.length ? slice.call(arguments, 2) : [];
+    for (i = 0, len = args.length; i < len; i++) {
+      arg = args[i];
+      if (this._isElementArgument(arg)) {
+        if (arg["ELEMENT"]["page_id"] !== this.currentPage.id) {
+          throw new Poltergeist.ObsoleteNode;
+        }
+      }
+    }
+    command = this.current_command;
+    cb = (function(_this) {
+      return function(result) {
+        return command.sendResponse(result);
+      };
+    })(this);
+    (ref = this.currentPage).evaluate_async.apply(ref, ["function() { " + script + " }", cb].concat(slice.call(args)));
+    return setTimeout((function(_this) {
+      return function() {
+        return command.sendError(new Poltergeist.ScriptTimeoutError);
+      };
+    })(this), max_wait * 1000);
+  };
+
+  Browser.prototype.execute = function() {
+    var arg, args, i, len, ref, script;
+    script = arguments[0], args = 2 <= arguments.length ? slice.call(arguments, 1) : [];
+    for (i = 0, len = args.length; i < len; i++) {
+      arg = args[i];
+      if (this._isElementArgument(arg)) {
+        if (arg["ELEMENT"]["page_id"] !== this.currentPage.id) {
+          throw new Poltergeist.ObsoleteNode;
+        }
+      }
+    }
+    (ref = this.currentPage).execute.apply(ref, ["function() { " + script + " }"].concat(slice.call(args)));
     return this.current_command.sendResponse(true);
   };
 
-  Browser.prototype.frameUrl = function(frame_name) {
-    return this.currentPage.frameUrl(frame_name);
+  Browser.prototype.frameUrlFor = function(frame_name) {
+    return this.currentPage.frameUrlFor(frame_name);
   };
 
   Browser.prototype.pushFrame = function(command, name, timeout) {
@@ -272,11 +336,11 @@ Poltergeist.Browser = (function() {
         name = frame.getAttribute('name');
       }
     }
-    frame_url = this.frameUrl(name);
+    frame_url = this.frameUrlFor(name);
     if (indexOf.call(this.currentPage.blockedUrls(), frame_url) >= 0) {
       return command.sendResponse(true);
     } else if (this.currentPage.pushFrame(name)) {
-      if (frame_url && (frame_url !== 'about:blank') && (this.currentPage.currentUrl() === 'about:blank')) {
+      if (frame_url && (frame_url !== 'about:blank') && (this.currentPage.frameUrl() === 'about:blank')) {
         this.currentPage.state = 'awaiting_frame_load';
         return this.currentPage.waitState('default', function() {
           return command.sendResponse(true);
@@ -304,8 +368,11 @@ Poltergeist.Browser = (function() {
     return this.pushFrame(this.current_command, name, timeout);
   };
 
-  Browser.prototype.pop_frame = function() {
-    return this.current_command.sendResponse(this.currentPage.popFrame());
+  Browser.prototype.pop_frame = function(pop_all) {
+    if (pop_all == null) {
+      pop_all = false;
+    }
+    return this.current_command.sendResponse(this.currentPage.popFrame(pop_all));
   };
 
   Browser.prototype.window_handles = function() {
@@ -358,18 +425,24 @@ Poltergeist.Browser = (function() {
     var page;
     page = this.getPageByHandle(handle);
     if (page) {
-      page.release();
+      page.close();
       return this.current_command.sendResponse(true);
     } else {
       return this.current_command.sendResponse(false);
     }
   };
 
-  Browser.prototype.mouse_event = function(page_id, id, name) {
+  Browser.prototype.mouse_event = function(page_id, id, name, keys, offset) {
     var command, event_page, last_mouse_event, node;
+    if (keys == null) {
+      keys = [];
+    }
+    if (offset == null) {
+      offset = {};
+    }
     node = this.node(page_id, id);
     this.currentPage.state = 'mouse_event';
-    last_mouse_event = node.mouseEvent(name);
+    last_mouse_event = node.mouseEvent(name, keys, offset);
     event_page = this.currentPage;
     command = this.current_command;
     return setTimeout(function() {
@@ -388,16 +461,16 @@ Poltergeist.Browser = (function() {
     }, 5);
   };
 
-  Browser.prototype.click = function(page_id, id) {
-    return this.mouse_event(page_id, id, 'click');
+  Browser.prototype.click = function(page_id, id, keys, offset) {
+    return this.mouse_event(page_id, id, 'click', keys, offset);
   };
 
-  Browser.prototype.right_click = function(page_id, id) {
-    return this.mouse_event(page_id, id, 'rightclick');
+  Browser.prototype.right_click = function(page_id, id, keys, offset) {
+    return this.mouse_event(page_id, id, 'rightclick', keys, offset);
   };
 
-  Browser.prototype.double_click = function(page_id, id) {
-    return this.mouse_event(page_id, id, 'doubleclick');
+  Browser.prototype.double_click = function(page_id, id, keys, offset) {
+    return this.mouse_event(page_id, id, 'doubleclick', keys, offset);
   };
 
   Browser.prototype.hover = function(page_id, id) {
@@ -447,36 +520,54 @@ Poltergeist.Browser = (function() {
   };
 
   Browser.prototype.send_keys = function(page_id, id, keys) {
-    var i, j, k, key, len, len1, len2, modifier_code, modifier_key, modifier_keys, sequence, target;
+    var target;
     target = this.node(page_id, id);
     if (!target.containsSelection()) {
       target.mouseEvent('click');
     }
+    this._send_keys_with_modifiers(keys);
+    return this.current_command.sendResponse(true);
+  };
+
+  Browser.prototype._send_keys_with_modifiers = function(keys, current_modifier_code) {
+    var i, j, k, key, len, len1, len2, modifier_code, modifier_key, modifier_keys, sequence;
+    if (current_modifier_code == null) {
+      current_modifier_code = 0;
+    }
     for (i = 0, len = keys.length; i < len; i++) {
       sequence = keys[i];
-      key = sequence.key != null ? this.currentPage.keyCode(sequence.key) : sequence;
+      if (sequence.key != null) {
+        if (!(key = this.currentPage.keyCode(sequence.key))) {
+          this.current_command.sendError(new Poltergeist.KeyError("Unknown key: " + sequence.key));
+          return;
+        }
+      } else if (sequence.keys != null) {
+        key = sequence.keys;
+      } else {
+        key = sequence;
+      }
       if (sequence.modifier != null) {
         modifier_keys = this.currentPage.keyModifierKeys(sequence.modifier);
-        modifier_code = this.currentPage.keyModifierCode(sequence.modifier);
+        modifier_code = this.currentPage.keyModifierCode(sequence.modifier) | current_modifier_code;
         for (j = 0, len1 = modifier_keys.length; j < len1; j++) {
           modifier_key = modifier_keys[j];
           this.currentPage.sendEvent('keydown', modifier_key);
         }
-        this.currentPage.sendEvent('keypress', key, null, null, modifier_code);
+        this._send_keys_with_modifiers([].concat(key), modifier_code);
         for (k = 0, len2 = modifier_keys.length; k < len2; k++) {
           modifier_key = modifier_keys[k];
           this.currentPage.sendEvent('keyup', modifier_key);
         }
       } else {
-        this.currentPage.sendEvent('keypress', key);
+        this.currentPage.sendEvent('keypress', key, null, null, current_modifier_code);
       }
     }
-    return this.current_command.sendResponse(true);
+    return true;
   };
 
-  Browser.prototype.render_base64 = function(format, arg) {
+  Browser.prototype.render_base64 = function(format, arg1) {
     var dimensions, encoded_image, full, ref, ref1, ref2, ref3, selector, window_scroll_position;
-    ref = arg != null ? arg : {}, full = (ref1 = ref.full) != null ? ref1 : false, selector = (ref2 = ref.selector) != null ? ref2 : null;
+    ref = arg1 != null ? arg1 : {}, full = (ref1 = ref.full) != null ? ref1 : false, selector = (ref2 = ref.selector) != null ? ref2 : null;
     window_scroll_position = this.currentPage["native"]().evaluate("function(){ return [window.pageXOffset, window.pageYOffset] }");
     dimensions = this.set_clip_rect(full, selector);
     encoded_image = this.currentPage.renderBase64(format);
@@ -488,9 +579,9 @@ Poltergeist.Browser = (function() {
     return this.current_command.sendResponse(encoded_image);
   };
 
-  Browser.prototype.render = function(path, arg) {
+  Browser.prototype.render = function(path, arg1) {
     var dimensions, format, full, options, quality, ref, ref1, ref2, ref3, ref4, ref5, selector, window_scroll_position;
-    ref = arg != null ? arg : {}, full = (ref1 = ref.full) != null ? ref1 : false, selector = (ref2 = ref.selector) != null ? ref2 : null, format = (ref3 = ref.format) != null ? ref3 : null, quality = (ref4 = ref.quality) != null ? ref4 : null;
+    ref = arg1 != null ? arg1 : {}, full = (ref1 = ref.full) != null ? ref1 : false, selector = (ref2 = ref.selector) != null ? ref2 : null, format = (ref3 = ref.format) != null ? ref3 : null, quality = (ref4 = ref.quality) != null ? ref4 : null;
     window_scroll_position = this.currentPage["native"]().evaluate("function(){ return [window.pageXOffset, window.pageYOffset] }");
     dimensions = this.set_clip_rect(full, selector);
     options = {};
@@ -550,8 +641,8 @@ Poltergeist.Browser = (function() {
     return this.current_command.sendResponse(true);
   };
 
-  Browser.prototype.network_traffic = function() {
-    return this.current_command.sendResponse(this.currentPage.networkTraffic());
+  Browser.prototype.network_traffic = function(type) {
+    return this.current_command.sendResponse(this.currentPage.networkTraffic(type));
   };
 
   Browser.prototype.clear_network_traffic = function() {
@@ -569,28 +660,45 @@ Poltergeist.Browser = (function() {
   };
 
   Browser.prototype.set_headers = function(headers) {
-    if (headers['User-Agent']) {
-      this.currentPage.setUserAgent(headers['User-Agent']);
+    return this.add_headers(headers, false, false);
+  };
+
+  Browser.prototype.add_headers = function(headers, local, keepExisting) {
+    var pages;
+    if (local == null) {
+      local = false;
     }
-    this.currentPage.setCustomHeaders(headers);
+    if (keepExisting == null) {
+      keepExisting = true;
+    }
+    pages = local ? [this.currentPage] : this.pages;
+    pages.forEach((function(_this) {
+      return function(page) {
+        var allHeaders, name, value;
+        allHeaders = keepExisting ? page.getCustomHeaders() : {};
+        for (name in headers) {
+          value = headers[name];
+          allHeaders[name] = value;
+        }
+        if (allHeaders['User-Agent']) {
+          page.setUserAgent(allHeaders['User-Agent']);
+        }
+        return page.setCustomHeaders(allHeaders);
+      };
+    })(this));
     return this.current_command.sendResponse(true);
   };
 
-  Browser.prototype.add_headers = function(headers) {
-    var allHeaders, name, value;
-    allHeaders = this.currentPage.getCustomHeaders();
-    for (name in headers) {
-      value = headers[name];
-      allHeaders[name] = value;
-    }
-    return this.set_headers(allHeaders);
-  };
-
-  Browser.prototype.add_header = function(header, permanent) {
-    if (!permanent) {
+  Browser.prototype.add_header = function(header, arg1) {
+    var permanent, ref;
+    permanent = (ref = arg1.permanent) != null ? ref : true;
+    if (permanent !== true) {
       this.currentPage.addTempHeader(header);
+      if (permanent === "no_redirect") {
+        this.currentPage.addTempHeaderToRemoveOnRedirect(header);
+      }
     }
-    return this.add_headers(header);
+    return this.add_headers(header, permanent !== true);
   };
 
   Browser.prototype.response_headers = function() {
@@ -636,6 +744,12 @@ Poltergeist.Browser = (function() {
     return this.current_command.sendResponse(true);
   };
 
+  Browser.prototype.set_page_settings = function(settings) {
+    this._page_settings = settings;
+    this.page.setSettings(this._page_settings);
+    return this.current_command.sendResponse(true);
+  };
+
   Browser.prototype.exit = function() {
     return phantom.exit();
   };
@@ -647,33 +761,31 @@ Poltergeist.Browser = (function() {
   };
 
   Browser.prototype.go_back = function() {
-    var command;
-    command = this.current_command;
     if (this.currentPage.canGoBack) {
-      this.currentPage.state = 'loading';
+      this.currentPage.state = 'wait_for_loading';
       this.currentPage.goBack();
-      return this.currentPage.waitState('default', function() {
-        return command.sendResponse(true);
-      });
+      return this._waitForHistoryChange();
     } else {
-      return command.sendResponse(false);
+      return this.current_command.sendResponse(false);
     }
   };
 
   Browser.prototype.go_forward = function() {
-    var command;
-    command = this.current_command;
     if (this.currentPage.canGoForward) {
-      this.currentPage.state = 'loading';
+      this.currentPage.state = 'wait_for_loading';
       this.currentPage.goForward();
-      return this.currentPage.waitState('default', function() {
-        return command.sendResponse(true);
-      });
+      return this._waitForHistoryChange();
     } else {
-      return command.sendResponse(false);
+      return this.current_command.sendResponse(false);
     }
   };
 
+  Browser.prototype.refresh = function() {
+    this.currentPage.state = 'wait_for_loading';
+    this.currentPage.reload();
+    return this._waitForHistoryChange();
+  };
+
   Browser.prototype.set_url_whitelist = function() {
     var wc, wildcards;
     wildcards = 1 <= arguments.length ? slice.call(arguments, 0) : [];
@@ -723,6 +835,23 @@ Poltergeist.Browser = (function() {
     return this.current_command.sendResponse(true);
   };
 
+  Browser.prototype._waitForHistoryChange = function() {
+    var command;
+    command = this.current_command;
+    return this.currentPage.waitState(['loading', 'default'], function(cur_state) {
+      if (cur_state === 'loading') {
+        return this.waitState('default', function() {
+          return command.sendResponse(true);
+        });
+      } else {
+        return command.sendResponse(true);
+      }
+    }, 0.5, function() {
+      this.state = 'default';
+      return command.sendResponse(true);
+    });
+  };
+
   Browser.prototype._wildcardToRegexp = function(wildcard) {
     wildcard = wildcard.replace(/[\-\[\]\/\{\}\(\)\+\.\\\^\$\|]/g, "\\$&");
     wildcard = wildcard.replace(/\*/g, ".*");
@@ -730,6 +859,10 @@ Poltergeist.Browser = (function() {
     return new RegExp(wildcard, "i");
   };
 
+  Browser.prototype._isElementArgument = function(arg) {
+    return typeof arg === "object" && typeof arg['ELEMENT'] === "object";
+  };
+
   return Browser;
 
 })();
