@@ -1,11 +1,12 @@
 var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
 
 Poltergeist.Connection = (function() {
-  function Connection(owner, port) {
+  function Connection(owner, port, host) {
     this.owner = owner;
     this.port = port;
+    this.host = host != null ? host : "127.0.0.1";
     this.commandReceived = bind(this.commandReceived, this);
-    this.socket = new WebSocket("ws://127.0.0.1:" + this.port + "/");
+    this.socket = new WebSocket("ws://" + this.host + ":" + this.port + "/");
     this.socket.onmessage = this.commandReceived;
     this.socket.onclose = function() {
       return phantom.exit();
