@@ -1,21 +1,23 @@
-var slice = [].slice,
+var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
+  slice = [].slice,
   indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
   hasProp = {}.hasOwnProperty;
 
 Poltergeist.WebPage = (function() {
   var command, delegate, fn1, fn2, i, j, len, len1, ref, ref1;
 
-  WebPage.CALLBACKS = ['onConsoleMessage', 'onError', 'onLoadFinished', 'onInitialized', 'onLoadStarted', 'onResourceRequested', 'onResourceReceived', 'onResourceError', 'onNavigationRequested', 'onUrlChanged', 'onPageCreated', 'onClosing'];
+  WebPage.CALLBACKS = ['onConsoleMessage', 'onError', 'onLoadFinished', 'onInitialized', 'onLoadStarted', 'onResourceRequested', 'onResourceReceived', 'onResourceError', 'onResourceTimeout', 'onNavigationRequested', 'onUrlChanged', 'onPageCreated', 'onClosing', 'onCallback'];
 
-  WebPage.DELEGATES = ['open', 'sendEvent', 'uploadFile', 'release', 'render', 'renderBase64', 'goBack', 'goForward'];
+  WebPage.DELEGATES = ['url', 'open', 'sendEvent', 'uploadFile', 'render', 'close', 'renderBase64', 'goBack', 'goForward', 'reload'];
 
-  WebPage.COMMANDS = ['currentUrl', 'find', 'nodeCall', 'documentSize', 'beforeUpload', 'afterUpload', 'clearLocalStorage'];
+  WebPage.COMMANDS = ['find', 'nodeCall', 'documentSize', 'beforeUpload', 'afterUpload', 'clearLocalStorage'];
 
   WebPage.EXTENSIONS = [];
 
-  function WebPage(_native) {
+  function WebPage(_native, settings) {
     var callback, i, len, ref;
     this._native = _native;
+    this._checkForAsyncResult = bind(this._checkForAsyncResult, this);
     this._native || (this._native = require('webpage').create());
     this.id = 0;
     this.source = null;
@@ -23,17 +25,24 @@ Poltergeist.WebPage = (function() {
     this.state = 'default';
     this.urlWhitelist = [];
     this.urlBlacklist = [];
-    this.frames = [];
     this.errors = [];
     this._networkTraffic = {};
     this._tempHeaders = {};
     this._blockedUrls = [];
     this._requestedResources = {};
+    this._responseHeaders = [];
+    this._tempHeadersToRemoveOnRedirect = {};
+    this._asyncResults = {};
+    this._asyncEvaluationId = 0;
+    this.setSettings(settings);
     ref = WebPage.CALLBACKS;
     for (i = 0, len = ref.length; i < len; i++) {
       callback = ref[i];
       this.bindCallback(callback);
     }
+    if (phantom.version.major < 2) {
+      this._overrideNativeEvaluate();
+    }
   }
 
   ref = WebPage.COMMANDS;
@@ -60,11 +69,25 @@ Poltergeist.WebPage = (function() {
     fn2(delegate);
   }
 
+  WebPage.prototype.setSettings = function(settings) {
+    var results, setting, value;
+    if (settings == null) {
+      settings = {};
+    }
+    results = [];
+    for (setting in settings) {
+      value = settings[setting];
+      results.push(this._native.settings[setting] = value);
+    }
+    return results;
+  };
+
   WebPage.prototype.onInitializedNative = function() {
     this.id += 1;
     this.source = null;
     this.injectAgent();
     this.removeTempHeaders();
+    this.removeTempHeadersForRedirect();
     return this.setScrollPosition({
       left: 0,
       top: 0
@@ -114,23 +137,20 @@ Poltergeist.WebPage = (function() {
     return true;
   };
 
+  WebPage.prototype.onCallbackNative = function(data) {
+    this._asyncResults[data['command_id']] = data['command_result'];
+    return true;
+  };
+
   WebPage.prototype.onResourceRequestedNative = function(request, net) {
-    var abort, blacklisted, ref2, useWhitelist, whitelisted;
-    useWhitelist = this.urlWhitelist.length > 0;
-    whitelisted = this.urlWhitelist.some(function(whitelisted_regex) {
-      return whitelisted_regex.test(request.url);
-    });
-    blacklisted = this.urlBlacklist.some(function(blacklisted_regex) {
-      return blacklisted_regex.test(request.url);
-    });
-    abort = false;
-    if (useWhitelist && !whitelisted) {
-      abort = true;
-    }
-    if (blacklisted) {
-      abort = true;
-    }
-    if (abort) {
+    var ref2;
+    this._networkTraffic[request.id] = {
+      request: request,
+      responseParts: [],
+      error: null
+    };
+    if (this._blockRequest(request.url)) {
+      this._networkTraffic[request.id].blocked = true;
       if (ref2 = request.url, indexOf.call(this._blockedUrls, ref2) < 0) {
         this._blockedUrls.push(request.url);
       }
@@ -138,14 +158,10 @@ Poltergeist.WebPage = (function() {
     } else {
       this.lastRequestId = request.id;
       if (this.normalizeURL(request.url) === this.redirectURL) {
+        this.removeTempHeadersForRedirect();
         this.redirectURL = null;
         this.requestId = request.id;
       }
-      this._networkTraffic[request.id] = {
-        request: request,
-        responseParts: [],
-        error: null
-      };
       this._requestedResources[request.id] = request.url;
     }
     return true;
@@ -161,6 +177,7 @@ Poltergeist.WebPage = (function() {
     }
     if (this.requestId === response.id) {
       if (response.redirectURL) {
+        this.removeTempHeadersForRedirect();
         this.redirectURL = this.normalizeURL(response.redirectURL);
       } else {
         this.statusCode = response.status;
@@ -179,6 +196,10 @@ Poltergeist.WebPage = (function() {
     return true;
   };
 
+  WebPage.prototype.onResourceTimeoutNative = function(request) {
+    return console.log("Resource request timed out for " + request.url);
+  };
+
   WebPage.prototype.injectAgent = function() {
     var extension, k, len2, ref2;
     if (this["native"]().evaluate(function() {
@@ -213,61 +234,74 @@ Poltergeist.WebPage = (function() {
   };
 
   WebPage.prototype.keyCode = function(name) {
+    if (name === "Ctrl") {
+      name = "Control";
+    }
     return this["native"]().event.key[name];
   };
 
   WebPage.prototype.keyModifierCode = function(names) {
     var modifiers;
     modifiers = this["native"]().event.modifier;
-    names = names.split(',').map((function(name) {
+    return names.split(',').map(function(name) {
       return modifiers[name];
-    }));
-    return names[0] | names[1];
+    }).reduce(function(n1, n2) {
+      return n1 | n2;
+    });
   };
 
   WebPage.prototype.keyModifierKeys = function(names) {
-    return names.split(',').map((function(_this) {
-      return function(name) {
-        return _this.keyCode(name.charAt(0).toUpperCase() + name.substring(1));
-      };
-    })(this));
+    var k, len2, name, ref2, results;
+    ref2 = names.split(',');
+    results = [];
+    for (k = 0, len2 = ref2.length; k < len2; k++) {
+      name = ref2[k];
+      if (!(name !== 'keypad')) {
+        continue;
+      }
+      name = name.charAt(0).toUpperCase() + name.substring(1);
+      results.push(this.keyCode(name));
+    }
+    return results;
   };
 
-  WebPage.prototype._waitState_until = function(state, callback, timeout, timeout_callback) {
-    if (this.state === state) {
-      return callback.call(this);
+  WebPage.prototype._waitState_until = function(states, callback, timeout, timeout_callback) {
+    var ref2;
+    if ((ref2 = this.state, indexOf.call(states, ref2) >= 0)) {
+      return callback.call(this, this.state);
     } else {
       if (new Date().getTime() > timeout) {
         return timeout_callback.call(this);
       } else {
         return setTimeout(((function(_this) {
           return function() {
-            return _this._waitState_until(state, callback, timeout, timeout_callback);
+            return _this._waitState_until(states, callback, timeout, timeout_callback);
           };
         })(this)), 100);
       }
     }
   };
 
-  WebPage.prototype.waitState = function(state, callback, max_wait, timeout_callback) {
-    var timeout;
+  WebPage.prototype.waitState = function(states, callback, max_wait, timeout_callback) {
+    var ref2, timeout;
     if (max_wait == null) {
       max_wait = 0;
     }
-    if (this.state === state) {
-      return callback.call(this);
+    states = [].concat(states);
+    if (ref2 = this.state, indexOf.call(states, ref2) >= 0) {
+      return callback.call(this, this.state);
     } else {
       if (max_wait !== 0) {
         timeout = new Date().getTime() + (max_wait * 1000);
         return setTimeout(((function(_this) {
           return function() {
-            return _this._waitState_until(state, callback, timeout, timeout_callback);
+            return _this._waitState_until(states, callback, timeout, timeout_callback);
           };
         })(this)), 100);
       } else {
         return setTimeout(((function(_this) {
           return function() {
-            return _this.waitState(state, callback);
+            return _this.waitState(states, callback);
           };
         })(this)), 100);
       }
@@ -280,8 +314,43 @@ Poltergeist.WebPage = (function() {
     return true;
   };
 
-  WebPage.prototype.networkTraffic = function() {
-    return this._networkTraffic;
+  WebPage.prototype.networkTraffic = function(type) {
+    var id, ref2, ref3, ref4, request, results, results1, results2;
+    switch (type) {
+      case 'all':
+        ref2 = this._networkTraffic;
+        results = [];
+        for (id in ref2) {
+          if (!hasProp.call(ref2, id)) continue;
+          request = ref2[id];
+          results.push(request);
+        }
+        return results;
+        break;
+      case 'blocked':
+        ref3 = this._networkTraffic;
+        results1 = [];
+        for (id in ref3) {
+          if (!hasProp.call(ref3, id)) continue;
+          request = ref3[id];
+          if (request.blocked) {
+            results1.push(request);
+          }
+        }
+        return results1;
+        break;
+      default:
+        ref4 = this._networkTraffic;
+        results2 = [];
+        for (id in ref4) {
+          if (!hasProp.call(ref4, id)) continue;
+          request = ref4[id];
+          if (!request.blocked) {
+            results2.push(request);
+          }
+        }
+        return results2;
+    }
   };
 
   WebPage.prototype.clearNetworkTraffic = function() {
@@ -315,10 +384,26 @@ Poltergeist.WebPage = (function() {
   };
 
   WebPage.prototype.title = function() {
+    return this["native"]().title;
+  };
+
+  WebPage.prototype.frameTitle = function() {
     return this["native"]().frameTitle;
   };
 
-  WebPage.prototype.frameUrl = function(frameNameOrId) {
+  WebPage.prototype.currentUrl = function() {
+    return this["native"]().url || this.runCommand('frameUrl');
+  };
+
+  WebPage.prototype.frameUrl = function() {
+    if (phantom.version.major > 2 || (phantom.version.major === 2 && phantom.version.minor >= 1)) {
+      return this["native"]().frameUrl || this.runCommand('frameUrl');
+    } else {
+      return this.runCommand('frameUrl');
+    }
+  };
+
+  WebPage.prototype.frameUrlFor = function(frameNameOrId) {
     var query;
     query = function(frameNameOrId) {
       var ref2;
@@ -387,6 +472,10 @@ Poltergeist.WebPage = (function() {
     }, selector);
   };
 
+  WebPage.prototype.getUserAgent = function() {
+    return this["native"]().settings.userAgent;
+  };
+
   WebPage.prototype.setUserAgent = function(userAgent) {
     return this["native"]().settings.userAgent = userAgent;
   };
@@ -395,6 +484,22 @@ Poltergeist.WebPage = (function() {
     return this["native"]().customHeaders;
   };
 
+  WebPage.prototype.getPermanentCustomHeaders = function() {
+    var allHeaders, name, ref2, ref3, value;
+    allHeaders = this.getCustomHeaders();
+    ref2 = this._tempHeaders;
+    for (name in ref2) {
+      value = ref2[name];
+      delete allHeaders[name];
+    }
+    ref3 = this._tempHeadersToRemoveOnRedirect;
+    for (name in ref3) {
+      value = ref3[name];
+      delete allHeaders[name];
+    }
+    return allHeaders;
+  };
+
   WebPage.prototype.setCustomHeaders = function(headers) {
     return this["native"]().customHeaders = headers;
   };
@@ -408,6 +513,26 @@ Poltergeist.WebPage = (function() {
     return this._tempHeaders;
   };
 
+  WebPage.prototype.addTempHeaderToRemoveOnRedirect = function(header) {
+    var name, value;
+    for (name in header) {
+      value = header[name];
+      this._tempHeadersToRemoveOnRedirect[name] = value;
+    }
+    return this._tempHeadersToRemoveOnRedirect;
+  };
+
+  WebPage.prototype.removeTempHeadersForRedirect = function() {
+    var allHeaders, name, ref2, value;
+    allHeaders = this.getCustomHeaders();
+    ref2 = this._tempHeadersToRemoveOnRedirect;
+    for (name in ref2) {
+      value = ref2[name];
+      delete allHeaders[name];
+    }
+    return this.setCustomHeaders(allHeaders);
+  };
+
   WebPage.prototype.removeTempHeaders = function() {
     var allHeaders, name, ref2, value;
     allHeaders = this.getCustomHeaders();
@@ -422,36 +547,35 @@ Poltergeist.WebPage = (function() {
   WebPage.prototype.pushFrame = function(name) {
     var frame_no;
     if (this["native"]().switchToFrame(name)) {
-      this.frames.push(name);
       return true;
-    } else {
-      frame_no = this["native"]().evaluate(function(frame_name) {
-        var f, frames, idx;
-        frames = document.querySelectorAll("iframe, frame");
-        return ((function() {
-          var k, len2, results;
-          results = [];
-          for (idx = k = 0, len2 = frames.length; k < len2; idx = ++k) {
-            f = frames[idx];
-            if ((f != null ? f['name'] : void 0) === frame_name || (f != null ? f['id'] : void 0) === frame_name) {
-              results.push(idx);
-            }
-          }
-          return results;
-        })())[0];
-      }, name);
-      if ((frame_no != null) && this["native"]().switchToFrame(frame_no)) {
-        this.frames.push(name);
-        return true;
-      } else {
-        return false;
-      }
     }
+    frame_no = this["native"]().evaluate(function(frame_name) {
+      var f, frames, idx;
+      frames = document.querySelectorAll("iframe, frame");
+      return ((function() {
+        var k, len2, results;
+        results = [];
+        for (idx = k = 0, len2 = frames.length; k < len2; idx = ++k) {
+          f = frames[idx];
+          if ((f != null ? f['name'] : void 0) === frame_name || (f != null ? f['id'] : void 0) === frame_name) {
+            results.push(idx);
+          }
+        }
+        return results;
+      })())[0];
+    }, name);
+    return (frame_no != null) && this["native"]().switchToFrame(frame_no);
   };
 
-  WebPage.prototype.popFrame = function() {
-    this.frames.pop();
-    return this["native"]().switchToParentFrame();
+  WebPage.prototype.popFrame = function(pop_all) {
+    if (pop_all == null) {
+      pop_all = false;
+    }
+    if (pop_all) {
+      return this["native"]().switchToMainFrame();
+    } else {
+      return this["native"]().switchToParentFrame();
+    }
   };
 
   WebPage.prototype.dimensions = function() {
@@ -491,55 +615,64 @@ Poltergeist.WebPage = (function() {
     return new Poltergeist.Node(this, id);
   };
 
-  WebPage.prototype.mouseEvent = function(name, x, y, button) {
+  WebPage.prototype.mouseEvent = function(name, x, y, button, modifiers) {
     if (button == null) {
       button = 'left';
     }
+    if (modifiers == null) {
+      modifiers = 0;
+    }
     this.sendEvent('mousemove', x, y);
-    return this.sendEvent(name, x, y, button);
+    return this.sendEvent(name, x, y, button, modifiers);
   };
 
   WebPage.prototype.evaluate = function() {
-    var args, fn;
+    var args, fn, ref2, result;
     fn = arguments[0], args = 2 <= arguments.length ? slice.call(arguments, 1) : [];
     this.injectAgent();
-    return JSON.parse(this.sanitize(this["native"]().evaluate("function() { return PoltergeistAgent.stringify(" + (this.stringifyCall(fn, args)) + ") }")));
+    result = (ref2 = this["native"]()).evaluate.apply(ref2, ["function() { var page_id = arguments[0]; var args = []; for(var i=1; i < arguments.length; i++){ if ((typeof(arguments[i]) == 'object') && (typeof(arguments[i]['ELEMENT']) == 'object')){ args.push(window.__poltergeist.get(arguments[i]['ELEMENT']['id']).element); } else { args.push(arguments[i]) } } var _result = " + (this.stringifyCall(fn, "args")) + "; return window.__poltergeist.wrapResults(_result, page_id); }", this.id].concat(slice.call(args)));
+    return result;
   };
 
-  WebPage.prototype.sanitize = function(potential_string) {
-    if (typeof potential_string === "string") {
-      return potential_string.replace("\n", "\\n").replace("\r", "\\r");
-    } else {
-      return potential_string;
-    }
+  WebPage.prototype.evaluate_async = function() {
+    var args, callback, cb, command_id, fn, ref2;
+    fn = arguments[0], callback = arguments[1], args = 3 <= arguments.length ? slice.call(arguments, 2) : [];
+    command_id = ++this._asyncEvaluationId;
+    cb = callback;
+    this.injectAgent();
+    (ref2 = this["native"]()).evaluate.apply(ref2, ["function(){ var page_id = arguments[0]; var args = []; for(var i=1; i < arguments.length; i++){ if ((typeof(arguments[i]) == 'object') && (typeof(arguments[i]['ELEMENT']) == 'object')){ args.push(window.__poltergeist.get(arguments[i]['ELEMENT']['id']).element); } else { args.push(arguments[i]) } } args.push(function(result){ result = window.__poltergeist.wrapResults(result, page_id); window.callPhantom( { command_id: " + command_id + ", command_result: result } ); }); " + (this.stringifyCall(fn, "args")) + "; return}", this.id].concat(slice.call(args)));
+    setTimeout((function(_this) {
+      return function() {
+        return _this._checkForAsyncResult(command_id, cb);
+      };
+    })(this), 10);
   };
 
   WebPage.prototype.execute = function() {
-    var args, fn;
+    var args, fn, ref2;
     fn = arguments[0], args = 2 <= arguments.length ? slice.call(arguments, 1) : [];
-    return this["native"]().evaluate("function() { " + (this.stringifyCall(fn, args)) + " }");
+    return (ref2 = this["native"]()).evaluate.apply(ref2, ["function() { for(var i=0; i < arguments.length; i++){ if ((typeof(arguments[i]) == 'object') && (typeof(arguments[i]['ELEMENT']) == 'object')){ arguments[i] = window.__poltergeist.get(arguments[i]['ELEMENT']['id']).element; } } " + (this.stringifyCall(fn)) + " }"].concat(slice.call(args)));
   };
 
-  WebPage.prototype.stringifyCall = function(fn, args) {
-    if (args.length === 0) {
-      return "(" + (fn.toString()) + ")()";
-    } else {
-      return "(" + (fn.toString()) + ").apply(this, PoltergeistAgent.JSON.parse(" + (JSON.stringify(JSON.stringify(args))) + "))";
+  WebPage.prototype.stringifyCall = function(fn, args_name) {
+    if (args_name == null) {
+      args_name = "arguments";
     }
+    return "(" + (fn.toString()) + ").apply(this, " + args_name + ")";
   };
 
   WebPage.prototype.bindCallback = function(name) {
-    var that;
-    that = this;
-    this["native"]()[name] = function() {
-      var result;
-      if (that[name + 'Native'] != null) {
-        result = that[name + 'Native'].apply(that, arguments);
-      }
-      if (result !== false && (that[name] != null)) {
-        return that[name].apply(that, arguments);
-      }
-    };
+    this["native"]()[name] = (function(_this) {
+      return function() {
+        var result;
+        if (_this[name + 'Native'] != null) {
+          result = _this[name + 'Native'].apply(_this, arguments);
+        }
+        if (result !== false && (_this[name] != null)) {
+          return _this[name].apply(_this, arguments);
+        }
+      };
+    })(this);
     return true;
   };
 
@@ -548,22 +681,20 @@ Poltergeist.WebPage = (function() {
     result = this.evaluate(function(name, args) {
       return __poltergeist.externalCall(name, args);
     }, name, args);
-    if (result !== null) {
-      if (result.error != null) {
-        switch (result.error.message) {
-          case 'PoltergeistAgent.ObsoleteNode':
-            throw new Poltergeist.ObsoleteNode;
-            break;
-          case 'PoltergeistAgent.InvalidSelector':
-            method = args[0], selector = args[1];
-            throw new Poltergeist.InvalidSelector(method, selector);
-            break;
-          default:
-            throw new Poltergeist.BrowserError(result.error.message, result.error.stack);
-        }
-      } else {
-        return result.value;
+    if ((result != null ? result.error : void 0) != null) {
+      switch (result.error.message) {
+        case 'PoltergeistAgent.ObsoleteNode':
+          throw new Poltergeist.ObsoleteNode;
+          break;
+        case 'PoltergeistAgent.InvalidSelector':
+          method = args[0], selector = args[1];
+          throw new Poltergeist.InvalidSelector(method, selector);
+          break;
+        default:
+          throw new Poltergeist.BrowserError(result.error.message, result.error.stack);
       }
+    } else {
+      return result != null ? result.value : void 0;
     }
   };
 
@@ -592,6 +723,123 @@ Poltergeist.WebPage = (function() {
     }
   };
 
+  WebPage.prototype._checkForAsyncResult = function(command_id, callback) {
+    if (this._asyncResults.hasOwnProperty(command_id)) {
+      callback(this._asyncResults[command_id]);
+      delete this._asyncResults[command_id];
+    } else {
+      setTimeout((function(_this) {
+        return function() {
+          return _this._checkForAsyncResult(command_id, callback);
+        };
+      })(this), 50);
+    }
+  };
+
+  WebPage.prototype._blockRequest = function(url) {
+    var blacklisted, useWhitelist, whitelisted;
+    useWhitelist = this.urlWhitelist.length > 0;
+    whitelisted = this.urlWhitelist.some(function(whitelisted_regex) {
+      return whitelisted_regex.test(url);
+    });
+    blacklisted = this.urlBlacklist.some(function(blacklisted_regex) {
+      return blacklisted_regex.test(url);
+    });
+    if (useWhitelist && !whitelisted) {
+      return true;
+    }
+    if (blacklisted) {
+      return true;
+    }
+    return false;
+  };
+
+  WebPage.prototype._overrideNativeEvaluate = function() {
+    return this._native.evaluate = function (func, args) {
+        function quoteString(str) {
+            var c, i, l = str.length, o = '"';
+            for (i = 0; i < l; i += 1) {
+                c = str.charAt(i);
+                if (c >= ' ') {
+                    if (c === '\\' || c === '"') {
+                        o += '\\';
+                    }
+                    o += c;
+                } else {
+                    switch (c) {
+                    case '\b':
+                        o += '\\b';
+                        break;
+                    case '\f':
+                        o += '\\f';
+                        break;
+                    case '\n':
+                        o += '\\n';
+                        break;
+                    case '\r':
+                        o += '\\r';
+                        break;
+                    case '\t':
+                        o += '\\t';
+                        break;
+                    default:
+                        c = c.charCodeAt();
+                        o += '\\u00' + Math.floor(c / 16).toString(16) +
+                            (c % 16).toString(16);
+                    }
+                }
+            }
+            return o + '"';
+        }
+
+        function detectType(value) {
+            var s = typeof value;
+            if (s === 'object') {
+                if (value) {
+                    if (value instanceof Array) {
+                        s = 'array';
+                    } else if (value instanceof RegExp) {
+                        s = 'regexp';
+                    } else if (value instanceof Date) {
+                        s = 'date';
+                    }
+                } else {
+                    s = 'null';
+                }
+            }
+            return s;
+        }
+
+        var str, arg, argType, i, l;
+        if (!(func instanceof Function || typeof func === 'string' || func instanceof String)) {
+            throw "Wrong use of WebPage#evaluate";
+        }
+        str = 'function() { return (' + func.toString() + ')(';
+        for (i = 1, l = arguments.length; i < l; i++) {
+            arg = arguments[i];
+            argType = detectType(arg);
+
+            switch (argType) {
+            case "object":      //< for type "object"
+            case "array":       //< for type "array"
+                str += JSON.stringify(arg) + ","
+                break;
+            case "date":        //< for type "date"
+                str += "new Date(" + JSON.stringify(arg) + "),"
+                break;
+            case "string":      //< for type "string"
+                str += quoteString(arg) + ',';
+                break;
+            default:            // for types: "null", "number", "function", "regexp", "undefined"
+                str += arg + ',';
+                break;
+            }
+        }
+        str = str.replace(/,$/, '') + '); }';
+        return this.evaluateJavaScript(str);
+    };;
+  };
+
   return WebPage;
 
 })();
